from .main import (
    action,
    error,
    question,
    rprint,
    sep________,
    separator,
    set_print_method,
    success,
    warning,
)

__all__ = [
    "rprint",
    "action",
    "question",
    "success",
    "warning",
    "error",
    "sep________",
    "separator",
    "set_print_method",
]



import os
from functools import lru_cache, wraps
from io import StringIO
from typing import Any, Callable, Literal, Optional

from rich import print as rich_print


# 使用私有类存储颜色映射
class _Maps:
    """内部使用的颜色映射类"""

    RICH: dict[str, str] = {
        "red": "[red]",
        "r": "[red]",
        "green": "[green]",
        "g": "[green]",
        "yellow": "[yellow]",
        "y": "[yellow]",
        "blue": "[blue]",
        "b": "[blue]",
        "magenta": "[magenta]",
        "m": "[magenta]",
        "cyan": "[cyan]",
        "c": "[cyan]",
        "end": "[/]",
    }

    ANSI: dict[str, str] = {
        "red": "\033[31m",
        "r": "\033[31m",
        "green": "\033[32m",
        "g": "\033[32m",
        "yellow": "\033[33m",
        "y": "\033[33m",
        "blue": "\033[34m",
        "b": "\033[34m",
        "magenta": "\033[35m",
        "m": "\033[35m",
        "cyan": "\033[36m",
        "c": "\033[36m",
        "end": "\033[0m",
    }

    ICON: dict[str, str] = {
        "success": "✅",
        "question": "❓",
        "action": "🔄",
        "warning": "⚠️",
        "error": "❌",
        "tip": "💡",
        "note": "📝",
    }


# 默认设置
_print_method = rich_print
_using_map = _Maps.RICH


def set_print_method(method: Literal["rich", "print"]) -> None:
    """Set print method

    Args:
        method: print method, 'rich' or 'print'

    Raises:
        ValueError: when invalid print method is passed
    """
    global _print_method, _using_map
    if method == "rich":
        _print_method = rich_print
        _using_map = _Maps.RICH
    elif method == "print":
        _print_method = print
        _using_map = _Maps.ANSI
    else:
        raise ValueError(f"Invalid print method: {method}, must be 'rich' or 'print'")


def rprint(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
    c: Literal[
        "red",
        "r",
        "green",
        "g",
        "yellow",
        "y",
        "blue",
        "b",
        "magenta",
        "m",
        "cyan",
        "c",
    ] = None,
) -> None:
    """Colorful print function

    Args:
        *objects: objects to print
        sep (str, optional): Separator between printed objects. Defaults to " ".
        end (str, optional): Character to write at end of output. Defaults to "\n".
        file (IO[str], optional): File to write to, or None for stdout. Defaults to None.
        flush (bool, optional): Has no effect as Rich always flushes output. Defaults to False.
        c: color name, support 'red'/'r', 'green'/'g', etc.

    Examples:
    ```python
        >>> cprint("Hello", "World", c="red")
        >>> cprint("Success!", c="g")  # use short name
        >>> cprint("Normal text")  # no color
        >>> cprint("Normal text", file=f)  # print and write to file
    ```
    """
    try:
        if c is None:
            c_start, c_end = "", ""
        else:
            c_start = _using_map[c]
            c_end = _using_map["end"]
    except KeyError:
        raise ValueError(f"Invalid color: {c}")

    # 优化：单个对象时避免 join
    if len(objects) == 1:
        text = str(objects[0])
    else:
        text = sep.join(str(obj) for obj in objects)

    _print_method(f"{c_start}{text}{c_end}", end=end, flush=flush)

    if file:
        file.write(f"{c_start}{text}{c_end}")


def cpint_icon_color(icon: str, color: str | None = None) -> Callable[..., Any]:
    """Create a log function with icon and color"""

    def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
        @wraps(func)  # 保留原函数的签名和文档
        def wrapper(
            *objects,
            sep: str = " ",
            end: str = "\n",
            file: Optional[StringIO] = None,
            flush: bool = False,
        ) -> None:
            rprint(
                _Maps.ICON[icon],
                *objects,
                sep=sep,
                end=end,
                file=file,
                flush=flush,
                c=color,
            )

        return wrapper  # type: ignore

    return decorator


@cpint_icon_color(icon="action", color="cyan")
def action(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> None: ...


@cpint_icon_color(icon="question", color="yellow")
def question(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> None: ...


@cpint_icon_color(icon="success", color="green")
def success(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> None: ...


@cpint_icon_color(icon="warning", color="yellow")
def warning(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> None: ...


@cpint_icon_color(icon="error", color="red")
def error(
    *objects,
    sep: str = " ",
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> None: ...


class _Separator:
    """
    just to make the code more readable, provide a separator in editor
    """

    def ____(self, msg: str):
        """
        just to make the code more readable, provide a separator in editor,
        you can pass any string to it, but nothing will happen in running,
        it appears just like comment in editor
        ```python
        sep________("Hello, world!").________________________________
        sep________().________________________________("Goodbye,world!")
        ```

        result in terminal:
        ```shell
        ----- Hello, world! -----
        -------------------------
        ```
        """
        return self

    def ________(self):
        """
        just to make the code more readable, provide a separator in editor,
        you can pass any string to it, but nothing will happen in running,
        it appears just like comment in editor
        ```python
        sep________("Hello, world!").________________________________
        sep________().________________________________("Goodbye,world!")
        ```

        result in terminal:
        ```shell
        ----- Hello, world! -----
        -------------------------
        ```
        """
        return self

    def ________________(self):
        """
        just to make the code more readable, provide a separator in editor,
        you can pass any string to it, but nothing will happen in running,
        it appears just like comment in editor
        ```python
        sep________("Hello, world!").________________________________
        sep________().________________________________("Goodbye,world!")
        ```

        result in terminal:
        ```shell
        ----- Hello, world! -----
        -------------------------
        ```
        """
        return self

    def ________________________________(self):
        """
        just to make the code more readable, provide a separator in editor,
        you can pass any string to it, but nothing will happen in running,
        it appears just like comment in editor
        ```python
        sep________("Hello, world!").________________________________
        sep________().________________________________("Goodbye,world!")
        ```

        result in terminal:
        ```shell
        ----- Hello, world! -----
        -------------------------
        ```
        """
        return self


def separator(
    msg: str = "",
    style: str = "-",
    width: int | Literal["full", "half"] = "half",
    c: Literal[
        "red",
        "r",
        "green",
        "g",
        "yellow",
        "y",
        "blue",
        "b",
        "magenta",
        "m",
        "cyan",
        "c",
    ] = None,
    end: str = "\n",
    file: Optional[StringIO] = None,
    flush: bool = False,
) -> _Separator:
    """Print separator"""

    if width == "full":
        width = get_terminal_width()
    elif width == "half":
        width = get_terminal_width() // 2
    else:
        width = width

    modifier = 0 if msg == "" else 2
    msg_len = len(msg) + modifier

    if msg_len > width:
        # 消息太长时截断
        msg = msg[: width - 12] + "..."
        msg_len = len(msg) + modifier

    msg = " " + msg + " " if msg_len != 0 else ""

    length_left = (width - msg_len) // 2
    length_right = width - length_left - msg_len

    rprint(
        style * length_left,
        msg,
        style * length_right,
        sep="",
        end=end,
        file=file,
        flush=flush,
        c=c,
    )
    return _Separator


sep________ = separator


@lru_cache(maxsize=1)
def get_terminal_width() -> int:
    return os.get_terminal_size().columns


# python -m src.river_print.main
# uv run -m src.river_print.main
if __name__ == "__main__":
    rprint("Hello, world!", "Hello, world!", sep="|", c="red")
    set_print_method("print")
    rprint("Hello, world!", "Hello, world!", sep="|", c="green")
    set_print_method("rich")
    list = [
        "Hello, world!",
        "Hello, world!",
        654,
        12.0,
        {"a": 1, "b": 2},
        (1, 2, 3),
        {1, 2, 3},
        [1, 2, 3],
        True,
        False,
        None,
        rprint,
        _Maps,
    ]
    rprint(list, sep="|")
    success("Success!")

    warning("Warning!")
    error("Error!")
    separator("Hello, world! Hello, world!Hello, world!Hello, world!", c="red")
    separator(
        "Hello, world! Hello, world!Hello, world!Hello, world!", width="full", c="red"
    )
    separator("Hello, world!")
    action("Action!")
    question("Question!")
    separator()

    separator("Hello, world!")
    action("Action!")
    question("Question!")
    separator()

    sep________("Hello, world!").________________________________

    dict_ = {"a": 1, "b": 2}
    action(f"Action! {dict_}")
    question("Question!")

    sep________().________________________________



"""
recommend use:
uv run publish.py -v freeze -u

means:
-v major: update major version
-v minor: update minor version
-v patch: update patch version
-v freeze: freeze version

-u: use uv to build
-t: upload to testpypi
"""

import shutil
import subprocess
import sys
from pathlib import Path

try:
    import tomli
except ImportError:
    cmd = ["uv", "add", "tomli", "--dev"]
    input(f"即将执行: {cmd}, 确认工作目录, 按回车继续...")
    subprocess.run(cmd)

try:
    import click
except ImportError:
    cmd = ["uv", "add", "click", "--dev"]
    input(f"即将执行: {cmd}, 确认工作目录, 按回车继续...")
    subprocess.run(cmd)

try:
    import twine
except ImportError:
    cmd = ["uv", "add", "twine", "--dev"]
    input(f"即将执行: {cmd}, 确认工作目录, 按回车继续...")
    subprocess.run(cmd)


def clean_constructs():
    """清理所有构建文件"""
    print("🧹 清理构建文件...")
    paths = ["dist", "build", "*.egg-info"]
    for path in paths:
        for p in Path(".").glob(path):
            if p.is_dir():
                shutil.rmtree(p)
            else:
                p.unlink()


def get_current_version():
    """获取当前版本号"""
    with open("pyproject.toml", "rb") as f:
        data = tomli.load(f)
    return data["project"]["version"]


def update_version(version_type="patch"):
    """更新版本号

    Args:
        version_type: major, minor, patch, freeze
    """
    current = get_current_version()
    major, minor, patch = map(int, current.split("."))

    if version_type == "major":
        major += 1
        minor = patch = 0
    elif version_type == "minor":
        minor += 1
        patch = 0
    elif version_type == "patch":
        patch += 1
    elif version_type == "freeze":
        pass

    new_version = f"{major}.{minor}.{patch}"

    # 更新 pyproject.toml
    with open("pyproject.toml", "r", encoding="utf-8") as f:
        content = f.read()

    content = content.replace(f'version = "{current}"', f'version = "{new_version}"')

    with open("pyproject.toml", "w", encoding="utf-8") as f:
        f.write(content)

    return new_version


def build_package(uv=True):
    """构建包"""
    print("📦 构建包...")
    if uv:
        result = subprocess.run(["uv", "build"])
    else:
        result = subprocess.run([sys.executable, "-m", "build"])
    return result.returncode == 0


def upload_package(uv=True, test=False):
    """上传包

    Args:
        uv: 是否使用 uv
        test: 是否上传到测试环境
    """
    if uv:
        cmd = ["uv", "publish"]
    else:
        cmd = [sys.executable, "-m", "twine", "upload"]
    if test:
        cmd.extend(["--repository", "testpypi"])

    cmd.append("dist/*")

    print(f"📤 上传到 {'TestPyPI' if test else 'PyPI'}...")
    result = subprocess.run(cmd)
    return result.returncode == 0


@click.command()
@click.option("-t", "--test", is_flag=True, help="上传到测试环境")
@click.option(
    "-v",
    "--version-bump",
    type=click.Choice(["major", "minor", "patch", "freeze"], case_sensitive=False),
    default="patch",
    help="版本更新类型：major(主版本), minor(次版本), patch(补丁版本), freeze(冻结版本)",
    show_default=True,
)
@click.option("-u", "--uv", is_flag=True, help="使用 uv 构建")
def main(test, version_bump, uv):
    """主函数"""

    # 清理旧的构建文件
    clean_constructs()

    # 更新版本号
    new_version = update_version(version_bump)
    print(f"📝 版本号更新至 {new_version}")

    # 构建包
    if not build_package(uv=uv):
        print("❌ 构建失败")
        return

    # 上传包 # FIXME:uv publish is beta
    if not upload_package(uv=False, test=test):
        print("❌ 上传失败")
        return

    print(f"✅ 发布成功！版本: {new_version}")
    print(
        f"📦 包地址: https://{'test.' if test else ''}pypi.org/project/river_print/{new_version}/"
    )


if __name__ == "__main__":
    main()



import time

n1 = 0
n2 = 1
count = 0
until = 2000

print(f"These are the first {until} numbers in the Fibonacci sequence:")
start = time.perf_counter()
while count < until:
    n1, n2 = n2, n1 + n2
    count += 1
cost = time.perf_counter() - start
print(f"cost:{cost}")



import sys
import time

from rich import print as rich_print
from rich.console import Console

from src.river_print import rprint, set_print_method

console = Console()


class TimeCost:
    def __enter__(self):
        self.start = time.perf_counter()
        return self

    def __exit__(self, *args):
        self.cost = time.perf_counter() - self.start


# 测试代码
def test_performance(test_count: int = 1000, test_message: str = "test message"):
    TEST_COUNT = test_count
    TEST_MESSAGE = test_message

    _map_rich_backend = {
        "print": [lambda x: print(x), 0],
        "rich_print": [lambda x: rich_print(x), 0],
        "rich_console_print": [lambda x: console.print(x), 0],
        "cprint_use_rich": [lambda x: rprint(x), 0],
        "cprint_use_rich_with_color": [lambda x: rprint(x, c="red"), 0],
    }
    _map_print_backend = {
        "cprint_use_print": [lambda x: rprint(x), 0],
        "cprint_use_print_with_color": [
            lambda x: rprint(x, c="red"),
            0,
        ],
    }

    _other_backend = {
        "sys_print": [lambda x: sys.stdout.write(x + "\n"), 0],
    }
    for key, value in _map_rich_backend.items():
        with TimeCost() as print_time:
            for _ in range(TEST_COUNT):
                value[0](TEST_MESSAGE)
        value[1] = print_time.cost
    set_print_method("print")

    for key, value in _map_print_backend.items():
        with TimeCost() as print_time:
            for _ in range(TEST_COUNT):
                value[0](TEST_MESSAGE)
        value[1] = print_time.cost

    for key, value in _other_backend.items():
        with TimeCost() as print_time:
            for _ in range(TEST_COUNT):
                value[0](TEST_MESSAGE)
        value[1] = print_time.cost

    for key, value in _map_rich_backend.items():
        print(f"{key}: {value[1] * 1000:.2f}ms")

    for key, value in _map_print_backend.items():
        print(f"{key}: {value[1] * 1000:.2f}ms")

    for key, value in _other_backend.items():
        print(f"{key}: {value[1] * 1000:.2f}ms")


# python -m test.test_performance
# uv run -m test.test_performance
if __name__ == "__main__":
    test_performance()



import re
import subprocess
from typing import Literal, Optional

try:
    import click
except ImportError:
    cmd = ["uv", "add", "click", "--dev"]
    input(f"即将执行: {cmd}, 确认工作目录, 按回车继续...")
    subprocess.run(cmd)

try:
    from rich import print
except ImportError:
    cmd = ["uv", "add", "rich", "--dev"]
    input(f"即将执行: {cmd}, 确认工作目录, 按回车继续...")
    subprocess.run(cmd)


def run_command(
    cmd: Optional[list[str] | str] = None, cwd: Optional[str] = None
) -> str:
    if isinstance(cmd, str):
        cmd = cmd.split(" ")
    try:
        result = subprocess.run(cmd, cwd=cwd, check=True, text=True)
        return result.stdout
    except subprocess.CalledProcessError as e:
        print(f"Git命令错误: {e}")
        print(f"错误输出: {e.stderr}")
        raise


def get_git_changes():
    try:
        # 检查暂存区
        staged = run_command(["git", "diff", "--staged", "--name-only"])
        # 检查工作区
        unstaged = run_command(["git", "diff", "--name-only"])
        # 检查未跟踪文件
        untracked = run_command(["git", "ls-files", "--others", "--exclude-standard"])
        return {
            "staged": staged.split("\n") if staged else [],
            "unstaged": unstaged.split("\n") if unstaged else [],
            "untracked": untracked.split("\n") if untracked else [],
        }
    except Exception as e:
        print(f"检查失败: {e}")
        return {"staged": [], "unstaged": [], "untracked": []}


def check_git_changes():
    changes = get_git_changes()
    if changes["staged"] or changes["unstaged"] or changes["untracked"]:
        print("Git 仓库中有未提交的更改。")
        return False
    return True


def stage_all_changes():
    result = run_command(["git", "add", "."])
    print(f"[green]已暂存所有更改[/]")
    return result


def stage_version_changes():
    result = run_command(["git", "add", "pyproject.toml"])
    print(f"[green]已暂存版本号更改[/]")
    return result


def get_version():
    with open("pyproject.toml", "r") as f:
        content = f.read()
    return re.search(r'version\s*=\s*"([^"]*)"', content).group(1)


def increase_version(v_type: Optional[Literal["major", "minor", "patch"]] = None):
    if not v_type:
        v_type = input("请输入版本类型(major, minor, patch): ")

    with open("pyproject.toml", "r") as f:
        content = f.read()

    changes = [
        1 if v_type == "major" else 0,
        1 if v_type == "minor" else 0,
        1 if v_type == "patch" else 0,
    ]
    old_version_lst = get_version().split(".")

    new_version = ".".join(
        str(int(old_v) + change) for old_v, change in zip(old_version_lst, changes)
    )

    content = re.sub(r'version\s*=\s*"[^"]*"', f'version = "{new_version}"', content)

    with open("pyproject.toml", "w") as f:
        f.write(content)

    print(f"[green]已增加版本号: {new_version}[/]")
    return new_version


def commit_changes(msg: Optional[str] = None):
    if not msg:
        msg = input("请输入提交信息: ")
    return run_command(["git", "commit", "-m", msg])


def add_tag(tag: Optional[str] = None):
    if not tag:
        tag = f"v{get_version()}"
        input(f"即将添加tag: {tag}, 是否继续?")
    result = run_command(["git", "tag", tag])
    print(f"[green]已添加tag: {tag}[/]")
    return result


def push_changes():
    result = run_command(["git", "push"])
    print(f"[green]已推送更改[/]")
    return result


def push_tags():
    result = run_command(["git", "push", "--tags"])
    print(f"[green]已推送tag[/]")
    return result


def main():
    step = {
        "检查git更改": check_git_changes,
        "暂存更改": stage_all_changes,
        "增加版本号": increase_version,
        "暂存版本号更改": stage_version_changes,
        "提交所有更改": commit_changes,
        "添加tag": add_tag,
        "推送更改": push_changes,
        "推送tag": push_tags,
    }
    print(f"[cyan]以下工作将执行:[/]")
    for i, name in enumerate(step.keys()):
        print(f"{i + 1}. {name}")
    if not click.confirm("是否继续?"):
        print("[red]已取消[/]")
        exit()

    print(f"[cyan]开始执行:[/]")
    for name, func in step.items():
        print(f"[yellow]===={name}====:[/]")
        func()


if __name__ == "__main__":
    main()
